@@ -1,3 +1,8 @@
+#' @include clhs.R utils.R
+#' @rdname clhs
+#' @method clhs SpatialPointsDataFrame
+#' @importFrom methods is
+#' @export
 clhs.SpatialPointsDataFrame <- function(
   x, # data
   ...
@@ -13,5 +18,3 @@ clhs.SpatialPointsDataFrame <- function(
     spl <- x[spl, ]
   spl
 }
-
-setMethod("clhs", "SpatialPointsDataFrame", clhs.SpatialPointsDataFrame)
